@rem ***************************************************************************
@rem Copyright (C) 2016 Mango Business Solutions Ltd, http://www.mango-solutions.com
@rem
@rem This program is free software: you can redistribute it and/or modify it under
@rem the terms of the GNU Affero General Public License as published by the
@rem Free Software Foundation, version 3.
@rem
@rem This program is distributed in the hope that it will be useful,
@rem but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@rem or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License
@rem for more details.
@rem
@rem You should have received a copy of the GNU Affero General Public License along
@rem with this program. If not, see <http://www.gnu.org/licenses/agpl-3.0.html>.
@rem ***************************************************************************
:: 
:: A Tool for creating SEE installation.
::
:: This file is a CLI for see-installer Maven module.

@ECHO OFF


SET HOME=%~dp0
SET MVN_HOME=%HOME%\lib\maven
SET M2_HOME=%MVN_HOME%
SET JAVA_HOME=%HOME%\lib\jre
SET MVN=%MVN_HOME%\bin\mvn.bat
SET INSTALLER_LOG=%HOME%\installer.log
SET CACHE_LOG=%HOME%\cache.log
SET OLD_PATH=%PATH%
SET MVN_OPTS=
SET MVN_SETTINGS=-s %MVN_HOME%\settings.xml
SET FINDCMD=C:\Windows\System32\find.exe
:: Preparing Environment
set PATH=%PATH:jre=jre-foo%
set PATH=%PATH:jdk=jdk-foo%
set PATH=%PATH:maven=maven-foo%
set PATH=%JAVA_HOME%\bin;%MVN_HOME%\bin;%PATH%


:: Internal Variables
SET modules=
SET configFile=
SET repository="http://sourceforge.net/projects/ddmore/files/install/SEE/Demonstrator-2.0.0/"
SET extra=
SET location=
SET clean=
SET location_param=
SET PARSE_ARG=
SET DEBUG=F
SET ERRORLEVEL=0
SET sep=:
SET ALL_MODULES=nonmem%sep%monolix%sep%simulx%sep%psn%sep%simcyp%sep%winbugs%sep%poped%sep%pfim
SET PSN=0
SET NONMEM=0
SET ARCH_32B=0
SET ARCH_64B=0
SET LOCATION=%HOME%\SEE

echo ===================================================================
echo SEE Version: Demonstrator-2.0.0
echo ===================================================================

:: Clean up log files
if exist %CACHE_LOG% (
	del %CACHE_LOG%
	)
if exist %INSTALLER_LOG% (
	del %INSTALLER_LOG%
	)


:: Parsing arguments
if [%1] == [] (
	set error_msg=Command not provided, one of "list, add, build" was expected.
	goto print-usage
	)

set command=%1
SHIFT

if [%command%] == [clean] (
	set clean=clean
	set command=%1
	SHIFT
)

if %DEBUG%==T echo "Argument parsing loop"
:Loop
IF [%1] == [] GOTO Continue
if %DEBUG%==T echo "Parsing "%1" in context %PARSE_ARG%"
set arg=%1
if %DEBUG%==T echo "Parsing "%arg%" in context %PARSE_ARG%"
if [%arg%] == [-extra] (
	set PARSE_ARG=extra
) else if [%arg%] == [-with] (
	set PARSE_ARG=modules
) else if [%arg%] == [full] (
	set modules=%ALL_MODULES%
	if %DEBUG%==T echo "Full installation requested: %modules%" 
	set PARSE_ARG=
) else if [%arg%] == [-modules] (
	set PARSE_ARG=modules
) else if [%arg%] == [-repository] (
	set PARSE_ARG=repository
) else if [%arg%] == [-configFile] (
	set PARSE_ARG=configFile
) else if [%arg%] == [-in] (
	set PARSE_ARG=location
) else if [%arg%] == [-to] (
	set PARSE_ARG=location
) else if [%arg%] == [-DEBUG] (
	set DEBUG=T 
) else if [%arg%] == [-VERBOSE] (
	set MVN_OPTS=-X
) else if [%arg%] == [-REUSE_EXISTING_MAVEN_SETTINGS] (
	set MVN_SETTINGS=
) else IF NOT [%PARSE_ARG%] == [] (
	if %DEBUG%==T echo "Parsing parameter value "%arg%" in context %PARSE_ARG%"
	if [%PARSE_ARG%] == [extra] (
		set extra=%extra% %arg%
	) else if [%PARSE_ARG%] == [modules] (
		if %DEBUG%==T echo "Parsing module name "%arg%""
		if NOT [%modules%] == [] (
			if %DEBUG%==T echo "Appending"
			set modules=%modules%%sep%%arg%
		) else (
			set modules=%arg%
		)
		if [%arg%] == [psn] SET PSN=1
		if [%arg%] == [nonmem] SET NONMEM=1
		if [%arg%] == [32b] SET ARCH_32B=1
		if [%arg%] == [64b] SET ARCH_64B=1
		
		if %DEBUG%==T echo "Modules "%modules%""
	) else if [%PARSE_ARG%] == [configFile] (
		set configFile=%arg%
	) else if [%PARSE_ARG%] == [repository] (
		set repository=%arg%
	) else if [%PARSE_ARG%] == [location] (
		set LOCATION=%arg%
	) else (
		set error_msg="Internal error: attempted to parse non-existing named argument"
		goto fail
	)
) else (
	set error_msg="Invalid command syntax: Unexpected named argument %arg%"
	goto print-usage
)
if %DEBUG%==T echo shifting
SHIFT
GOTO Loop
:Continue

if %DEBUG%==T echo "Interpret arguments"

:: Interpreting arguments
if NOT [%LOCATION%] == [] (
	SET location_param=-Doutput.see="%LOCATION%"
)

if NOT [%repository%] == [] (
	SET repository_param=-Dbuild.configuration.file.remote=%repository%
)

if NOT [%configFile%] == [] (
	SET config_param=-Dbuild.configuration.file.name=%configFile%
)

if %ARCH_32B% EQU 1 (
	if %ARCH_64B% EQU 1 (
		set error_msg=Only one '32b' or '64b' can be selected, not both. 
		goto fail
	)
	SET modules=%modules%%sep%!64b
) else if %ARCH_64B% EQU 1 (
	if %ARCH_32B% EQU 1 (
		set error_msg=Only one '32b' or '64b' can be selected, not both.
		goto fail
	)
	SET modules=%modules%%sep%!32b
)

if %DEBUG%==T echo Switch for %command%

if "%command%" == "list" (
	goto list-modules
) else if "%command%" == "add" (
	set main_profile=add
	if [%modules%] == [] (
		echo Expected list of modules as a second parameter.
		goto list-modules
	)
	
	if [%location%] == [] (
		set error_msg=You must specify location of existing SEE installation using '-to' parameter 
		goto fail
	)
	:: we are adding modules now, we must deselect default profiles
	SET modules=%modules%%sep%!mdl-ide%sep%!see%sep%!core-modules
	goto refresh-cache
) else if "%command%" == "build" (
	set main_profile=new
	if [%modules%] == [] (
		echo Building SEE with just base modules, no third party plugins will be included
	)
	goto refresh-cache

) else if [%clean%] == [clean] (
	goto clean
) else (
	set error_msg="Invalid command syntax: unsupported command - %command%"
	goto fail
)

:clean
	SET log_param=-l %CACHE_LOG%
	if %DEBUG%==T (
		SET log_param=
	)
	CALL %MVN% %MVN_SETTINGS% %MVN_OPTS% -f %HOME%\lib\see-installer\cache.xml %clean% %log_param%
	if ERRORLEVEL 1 (
		if %DEBUG%==T (
	    	set error_msg=Cleaning cache failed. See console output for details.
		) else (
	    	set error_msg=Cleaning cache failed. Refer to %CACHE_LOG% for details.
	    )
	    goto fail
	)
	goto success
	
:list-modules
	echo There are the following modules available:
	echo * nonmem
	echo * monolix
	echo * simulx
	echo * psn
	echo * simcyp
	echo The following modules are always installed:
	echo * fis
	echo * mif
	echo * cts
    echo The following modules are installed by default:
    echo * mdl-ide
	goto end

:print-usage
	type %HOME%\USAGE.txt
	goto fail

:refresh-cache
	echo Refreshing Cache
:: no proper boolean logic operators
	SET NONMEM_TRACE_FILE=%LOCATION%\.installer\nonmem-see-bundle.trace
	if %DEBUG%==T echo %NONMEM_TRACE_FILE%
	
	if %PSN% EQU 1 (
		if %DEBUG%==T echo "Requested PsN plugin installation"
		If %NONMEM% EQU 0 (
			if %DEBUG%==T echo "No nonmem selected"
:: We need to handle this in here, no way of coding this rule in Maven
			if NOT EXIST %NONMEM_TRACE_FILE% (
				if %DEBUG%==T echo "Nonmem plugin is missing"
				SET modules=%modules%%sep%nonmem
			) else (
				if %DEBUG%==T echo "Nonmem plugin already installed"
			)
		)
	) else (
		if %DEBUG%==T echo "PsN plugin installation not requested"
	)
	if NOT [%modules%] == [] (
		SET modules_cache=-P %modules::=,%
	) else (
		SET modules_cache=
	)
	SET log_param=-l %CACHE_LOG%
	if %DEBUG%==T (
		SET log_param=
		echo "CALL %MVN% %MVN_SETTINGS% %MVN_OPTS% -f %HOME%\lib\see-installer\cache.xml %clean% package %modules_cache% %repository_param% %config_param% %extra% %log_param%"
	)
	CALL %MVN% %MVN_SETTINGS% %MVN_OPTS% -f %HOME%\lib\see-installer\cache.xml %clean% package %modules_cache% %extra% %repository_param% %config_param% %log_param%
		
	if ERRORLEVEL 1 (
		if %DEBUG%==T (
	    	set error_msg=Building of SEE failed. See console output for details.
		) else (
	    	set error_msg=Building of SEE failed. Refer to %CACHE_LOG% for details.
	    )
	    goto fail
	)

:install
	echo Performing Installation
	if NOT [%modules%] == [] (
		SET modules=,%modules::=,%
	)
	SET log_param=-l %INSTALLER_LOG%
	if %DEBUG%==T (
		SET log_param=
		echo "CALL %MVN% %MVN_SETTINGS% %MVN_OPTS% -f %HOME%\lib\see-installer\pom.xml install -P%main_profile%%modules% %location_param% %repository_param% %config_param% %extra% %log_param%"
	)
	echo This will take a while (even 2h if full installation was requested)
	echo See %INSTALLER_LOG% for more detailed progress
	CALL %MVN% %MVN_SETTINGS% %MVN_OPTS% -f %HOME%\lib\see-installer\pom.xml install -P%main_profile%%modules% %location_param% %repository_param% %config_param% %extra% %log_param%

	if ERRORLEVEL 1 (
		if %DEBUG%==T (
	    	set error_msg=Building of SEE failed. See console output for details.
		) else (
	    	set error_msg=Building of SEE failed. Refer to %INSTALLER_LOG% for details.
	    )
	    goto fail
	)

:success
	echo ----------
	echo INSTALLATION SUCCESS
	echo ----------
	goto end

:fail
	echo ----------
	echo INSTALLATION FAILURE
    echo %error_msg% 
	echo ----------
	EXIT /B 1
:end
	set PATH=%OLD_PATH%
	EXIT /B 0